@@ -1,5 +1,5 @@
 #
-# Copyright 2018-2020 IBM Corporation
+# Copyright 2018-2021 Elyra Authors
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -20,10 +20,10 @@ import time
 from abc import ABC, abstractmethod
 from elyra.pipeline import PipelineProcessor, PipelineProcessorResponse, Operation
 from elyra.util.path import get_absolute_path
-from notebook.gateway.managers import GatewayClient
-from subprocess import run
+from jupyter_server.gateway.managers import GatewayClient
+from subprocess import run, CalledProcessError, PIPE
 from traitlets import log
-from typing import Dict
+from typing import Dict, List
 
 
 class LocalPipelineProcessor(PipelineProcessor):
@@ -42,12 +42,14 @@ class LocalPipelineProcessor(PipelineProcessor):
     _type = 'local'
 
     def __init__(self, root_dir, **kwargs):
-        super(LocalPipelineProcessor, self).__init__(root_dir, **kwargs)
+        super().__init__(root_dir, **kwargs)
         notebook_op_processor = NotebookOperationProcessor(self.root_dir)
         python_op_processor = PythonScriptOperationProcessor(self.root_dir)
+        r_op_processor = RScriptOperationProcessor(self.root_dir)
         self._operation_processor_registry = {
             notebook_op_processor.operation_name: notebook_op_processor,
-            python_op_processor.operation_name: python_op_processor
+            python_op_processor.operation_name: python_op_processor,
+            r_op_processor.operation_name: r_op_processor,
         }
 
     @property
@@ -66,7 +68,7 @@ class LocalPipelineProcessor(PipelineProcessor):
         t0_all = time.time()
 
         # Sort operations based on dependency graph (topological order)
-        operations = LocalPipelineProcessor._sort_operations(pipeline.operations)
+        operations = PipelineProcessor._sort_operations(pipeline.operations)
         for operation in operations:
             try:
                 t0 = time.time()
@@ -76,44 +78,26 @@ class LocalPipelineProcessor(PipelineProcessor):
                                        operation_name=operation.name,
                                        duration=(time.time() - t0))
             except Exception as ex:
-                raise RuntimeError(f'Error processing operation {operation.name}.') from ex
+                raise RuntimeError(f'Error processing operation {operation.name} {str(ex)}') from ex
 
         self.log_pipeline_info(pipeline.name, "pipeline processed", duration=(time.time() - t0_all))
 
-        return PipelineProcessorResponse('', '', '')
+        return LocalPipelineProcessorResponse()
 
     def export(self, pipeline, pipeline_export_format, pipeline_export_path, overwrite):
         raise NotImplementedError('Local pipelines does not support export functionality')
 
-    @staticmethod
-    def _sort_operations(operations_by_id: dict) -> list:
-        """
-        Sort the list of operations based on its dependency graph
-        """
-        ordered_operations = []
 
-        for operation in operations_by_id.values():
-            LocalPipelineProcessor._sort_operation_dependencies(operations_by_id,
-                                                                ordered_operations,
-                                                                operation)
+class LocalPipelineProcessorResponse(PipelineProcessorResponse):
 
-        return ordered_operations
+    _type = 'local'
 
-    @staticmethod
-    def _sort_operation_dependencies(operations_by_id: dict, ordered_operations: list, operation: Operation) -> None:
-        """
-        Helper method to the main sort operation function
-        """
-        # Optimization: check if already processed
-        if operation not in ordered_operations:
-            # process each of the dependencies that needs to be executed first
-            for parent_operation_id in operation.parent_operations:
-                parent_operation = operations_by_id[parent_operation_id]
-                if parent_operation not in ordered_operations:
-                    LocalPipelineProcessor._sort_operation_dependencies(operations_by_id,
-                                                                        ordered_operations,
-                                                                        parent_operation)
-            ordered_operations.append(operation)
+    def __init__(self):
+        super().__init__('', '', '')
+
+    @property
+    def type(self):
+        return self._type
 
 
 class OperationProcessor(ABC):
@@ -134,8 +118,10 @@ class OperationProcessor(ABC):
 
 class FileOperationProcessor(OperationProcessor):
 
+    MAX_ERROR_LEN: int = 80
+
     def __init__(self, root_dir: str):
-        super(FileOperationProcessor, self).__init__()
+        super().__init__()
         self._root_dir = root_dir
 
     @property
@@ -146,7 +132,7 @@ class FileOperationProcessor(OperationProcessor):
     def process(self, operation: Operation):
         raise NotImplementedError
 
-    def get_valid_filepath(self, op_filename):
+    def get_valid_filepath(self, op_filename: str) -> str:
         filepath = get_absolute_path(self._root_dir, op_filename)
         if not os.path.exists(filepath):
             raise FileNotFoundError(f'Could not find {filepath}')
@@ -154,13 +140,38 @@ class FileOperationProcessor(OperationProcessor):
             raise ValueError(f'Not a file: {filepath}')
         return filepath
 
+    def log_and_raise(self, file_name: str, ex: Exception) -> None:
+        """Log and raise the exception that occurs when processing file_name.
+
+        If the exception's message is longer than MAX_ERROR_LEN, it will be
+        truncated with an ellipses (...) when raised.  The complete message
+        will be logged.
+        """
+        self.log.error(f'Error executing {file_name}: {str(ex)}')
+        truncated_msg = FileOperationProcessor._truncate_msg(str(ex))
+        raise RuntimeError(f'({file_name}): {truncated_msg}') from ex
+
+    @staticmethod
+    def _truncate_msg(msg: str) -> str:
+        """Truncates the msg string to be less that MAX_ERROR_LEN.
+
+        If msg is longer than MAX_ERROR_LEN, the first space is found from the right,
+        then ellipses (...) are appended to that location so that they don't appear
+        in the middle of a word.  As a result, the truncation could result in lengths
+        less than the max+2.
+        """
+        if len(msg) < FileOperationProcessor.MAX_ERROR_LEN:
+            return msg
+        # locate the first whitespace from the 80th character and truncate from there
+        last_space = msg.rfind(' ', 0, FileOperationProcessor.MAX_ERROR_LEN)
+        if last_space >= 0:
+            return msg[:last_space] + "..."
+        return msg[:FileOperationProcessor.MAX_ERROR_LEN]
+
 
 class NotebookOperationProcessor(FileOperationProcessor):
     _operation_name = 'execute-notebook-node'
 
-    def __init__(self, root_dir: str):
-        super(NotebookOperationProcessor, self).__init__(root_dir)
-
     def process(self, operation: Operation):
         filepath = self.get_valid_filepath(operation.filename)
 
@@ -180,7 +191,9 @@ class NotebookOperationProcessor(FileOperationProcessor):
         additional_kwargs['engine_name'] = "ElyraEngine"
         additional_kwargs['cwd'] = file_dir  # For local operations, papermill runs from this dir
         additional_kwargs['kernel_cwd'] = file_dir
-        additional_kwargs['kernel_env'] = operation.env_vars_as_dict()
+        envs = os.environ.copy()  # Make sure this process's env is "available" in the kernel subprocess
+        envs.update(operation.env_vars_as_dict())
+        additional_kwargs['kernel_env'] = envs
         if GatewayClient.instance().gateway_enabled:
             additional_kwargs['kernel_manager_class'] = 'elyra.pipeline.http_kernel_manager.HTTPKernelManager'
 
@@ -191,19 +204,25 @@ class NotebookOperationProcessor(FileOperationProcessor):
                 filepath,
                 **additional_kwargs
             )
+        except papermill.PapermillExecutionError as pmee:
+            self.log.error(f'Error executing {file_name} in cell {pmee.exec_count}: ' +
+                           f'{str(pmee.ename)} {str(pmee.evalue)}')
+            raise RuntimeError(f'({file_name}) in cell {pmee.exec_count}: ' +
+                               f'{str(pmee.ename)} {str(pmee.evalue)}') from pmee
         except Exception as ex:
-            raise RuntimeError(f'Internal error executing {filepath}: {ex}') from ex
+            self.log_and_raise(file_name, ex)
 
         t1 = time.time()
         duration = (t1 - t0)
         self.log.debug(f'Execution of {file_name} took {duration:.3f} secs.')
 
 
-class PythonScriptOperationProcessor(FileOperationProcessor):
-    _operation_name = 'execute-python-node'
+class ScriptOperationProcessor(FileOperationProcessor):
 
-    def __init__(self, root_dir):
-        super(PythonScriptOperationProcessor, self).__init__(root_dir)
+    _script_type: str = None
+
+    def get_argv(self, filepath) -> List[str]:
+        raise NotImplementedError
 
     def process(self, operation: Operation):
         filepath = self.get_valid_filepath(operation.filename)
@@ -211,18 +230,43 @@ class PythonScriptOperationProcessor(FileOperationProcessor):
         file_dir = os.path.dirname(filepath)
         file_name = os.path.basename(filepath)
 
-        self.log.debug(f'Processing python script: {filepath}')
+        self.log.debug(f'Processing {self._script_type} script: {filepath}')
 
-        argv = ['python3', filepath, '--PYTHONHOME', file_dir]
+        argv = self.get_argv(filepath)
 
-        envs = os.environ  # Make sure this process's env is "available" in subprocess
+        envs = os.environ.copy()  # Make sure this process's env is "available" in subprocess
         envs.update(operation.env_vars_as_dict())
         t0 = time.time()
         try:
-            run(argv, cwd=file_dir, env=envs, check=True)
+            run(argv, cwd=file_dir, env=envs, check=True, stderr=PIPE)
+        except CalledProcessError as cpe:
+            error_msg = str(cpe.stderr.decode())
+            self.log.error(f'Error executing {file_name}: {error_msg}')
+
+            error_trim_index = error_msg.rfind('\n', 0, error_msg.rfind('Error'))
+            if error_trim_index != -1:
+                raise RuntimeError(f'({file_name}): {error_msg[error_trim_index:].strip()}') from cpe
+            else:
+                raise RuntimeError(f'({file_name})') from cpe
         except Exception as ex:
-            raise RuntimeError(f'Internal error executing {filepath}: {ex}') from ex
+            self.log_and_raise(file_name, ex)
 
         t1 = time.time()
         duration = (t1 - t0)
         self.log.debug(f'Execution of {file_name} took {duration:.3f} secs.')
+
+
+class PythonScriptOperationProcessor(ScriptOperationProcessor):
+    _operation_name = 'execute-python-node'
+    _script_type = 'Python'
+
+    def get_argv(self, file_path) -> List[str]:
+        return ['python3', file_path, '--PYTHONHOME', os.path.dirname(file_path)]
+
+
+class RScriptOperationProcessor(ScriptOperationProcessor):
+    _operation_name = 'execute-r-node'
+    _script_type = 'R'
+
+    def get_argv(self, file_path) -> List[str]:
+        return ['Rscript', file_path]
